/*
-------------------------------------------------------------------
DSudoku 1.0: a WinS Sudoku game using Sudoku Sensei 1.03
Copyright (C) 2005 Patrick Huy

SUDOKU SENSEI 1.03
Copyright (C) 2005  Martin Knoblauch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

-------------------------------------------------------------------
*/
#include "CtlSudoku.h"

//Taken from sudoko sensei demo3-minimal-creator
int my_random (int num)
{
	double d;
	int r;
	
	d = rand () / (double) RAND_MAX;
	r = (int) (d*num);
	
	return r<0 ? 0 : r>=num ? num-1 : r;
}

CtlSudoku::CtlSudoku (Bbox b) :
	Control (b), focus (-1,-1), help (false), done(false)
{
	srand (IPC->rtc_seconds + IPC->rtc_minutes *  60 + IPC->rtc_hours * 60 * 60 + IPC->rtc_day * 60 * 60 * 24 + IPC->rtc_month * 60 * 60 * 24 * 31 + (IPC->rtc_year + 2000) * 60 * 60 * 24 * 365);
	this->WS_MSG_SUDOKU_FOCUS_CHANGED = gWinS.CreateCustomMessage();
	this->WS_MSG_SUDOKU_CLEARED = gWinS.CreateCustomMessage(); //why does this return WS_MSG_CHECKBOXPRESSED?
	memset (field, 0, 9*9*sizeof(SudokuWu));
	for (int i = 0; i < 9; i++)
	for (int j = 0; j < 9; j++)
	{
		//field[i][j].visibleValue = j;
		field[i][j].userSet = true;
	}
}

void CtlSudoku::FocusField (int _x, int _y)
{
	if (focus.x != _x || focus.y != _y)
	{
		focus.x = _x;
		focus.y = _y;
		UpdateAll();
		gWinS.Draw();
		Screen.Show();
		if(_ancestor)
			SendMessage(this->WS_MSG_SUDOKU_FOCUS_CHANGED, _ancestor);
	}
}

//this is demo3-minimal-creator of sudoku sensei
void CtlSudoku::Create ()
{
	t_board_p b;
	e_state state;
	int i, j, c, s, n, m;
	int x,y;
	int cells[81], symbols[81];
	
	//srand ((unsigned)time(NULL));  /* Use clock as random seed */
	
	b = ConstructBasicBoard (3,       /* (3*3)*(3*3) == 81 cells */
	FALSE);  /* No diagonals */
	
	if (b)
	{
		
		n = 0;   /* n: number of symbols fixed */
		
		do
		{
			do                                      /* Choose a cell */
				c = my_random (81);                   /* at random */
			while (GetNumPossValuesOfCell(b,c)==1); /* c: cell number */
			
			s = my_random (                       /* Choose one of */
			GetNumPossValuesOfCell (b, c)); /* the possible */
			/* values */
			
			m = GetPossValuesOfCell (b, c);  /* Mask with possible */
			/* values */
			for (i=j=0; i<9; i++)
			if ((1<<i) & m)        /* Search for the chosen */
			{                      /* possible value in the mask */
				if (j==s)
				{
					s = i;             /* s: chosen symbol number */
					break;
				}
				
				j ++;
			}
			
			SetSymbolInCell (b, s, c);  /* Set it, and then */
			/* try to solve */
			Solve (b, 0, FALSE);
			
			state = GetState (b);
			
			if (state==impossible)   /* This seldom happens, but.. */
			{
				CleanBoard (b);        /* Start from scratch */
				n = 0;
				continue;
			}
			
			symbols[n] = s;   /* The choice was right */
			cells[n] = c;     /* Store cell-symbol pair */
			n ++;
		}
		while (state!=solved);   /* Repeat until solved */
		
		//printf ("Solution:\n");
		//PrintBoard (b, stdout, NULL, NULL);
		
		for (x = 0; x < 9; x++)
		for (y = 0; y < 9; y++)
		{
			field[x][y].realValue = GetSymbolOfCell(b, GetCellNumber (b, x, y)) + 1;
			field[x][y].userSet = true;
		}
		
		
		CleanBoard (b);          /* Clean the board and set */
		/* all the n symbols again */
		for (i=0; i<n; i++)
		  SetSymbolInCell (b, symbols[i],
							cells[i]);
		for (x = 0; x < 9; x++)
		for (y = 0; y < 9; y++)
		{
			field[x][y].visibleValue = GetSymbolOfCell(b, GetCellNumber (b, x, y)) + 1;
			if (field[x][y].visibleValue > 0 && field[x][y].visibleValue < 10) {
				field[x][y].userSet = false;
			} else {
				field[x][y].visibleValue = 0;
			}
		}
		
		this->done = false;
		UpdateAll();
		gWinS.Draw();
		Screen.Show();
		
		DestroyBoard (b);
	}
}

bool CtlSudoku::setFocusedTo (int newVal)
{
	if (focus.x < 0 || done) return false;
	if (!field[focus.x][focus.y].userSet || newVal < 0 || newVal > 9) return false;
	field[focus.x][focus.y].visibleValue = newVal;
	
	this->done = true;
	for (int i = 0; i < 9; i++)
	for (int j = 0; j < 9; j++)
		if (field[i][j].realValue != field[i][j].visibleValue) this->done = false;
	if (this->done && _ancestor)
		SendMessage(this->WS_MSG_SUDOKU_CLEARED, _ancestor);
		
	UpdateAll();
	gWinS.Draw();
	Screen.Show();
	return true;
}

void CtlSudoku::HandleMessages()
{
	WS_MSG msg;
	s32 x,y;
	
	FORALLMESSAGES(msg)
	{
		switch(msg.id)
		{
			case WS_MSG_POINTERINPUT:
				if (done) break;
				gWinS.GetTouch(x,y);
				while(gWinS.InputReceived()) { //Let's find out where s.o. clicked o.o (please note that this is not the most efficient solution)
					for (int i = 0; i < 9; i++)
					for (int j = 0; j < 9; j++)
					{
						if (fieldBbox(i,j).IsContained (x,y)) { FocusField (i,j); goto noLoop; } //there is no other way I know of to break out of 2 loops at once
					}
					noLoop:
					gWinS.GetTouch(x,y);
				}
				break;
		}
	}
	Control::HandleMessages();
}


/*
c = 2;
b = 1;
9*a + 6*b + 4*c = h

9*a + 6 + 8 = h
a = (h - 14) / 9
*/
void CtlSudoku::_DrawBackground()
{
	Screen.Fill(_bbox.Inside(2), WS_COLOR_WHITE);
	Screen.DrawRectangle(_bbox, WS_COLOR_BLACK);
	Screen.DrawRectangle(_bbox.Inside(1), WS_COLOR_BLACK);
	
	int boxHeight = (_bbox.ydim - 14) / 9;
	int boxWidth = (_bbox.xdim - 14) / 9;
	int y = _bbox.y + 2 + boxHeight;
	for (int i = 1; i < 9; i++)
	{
		Screen.DrawHLine (_bbox.x + 2, _bbox.x + _bbox.xdim  - 2, y, WS_COLOR_BLACK);
		if (i % 3 == 0) {
			y++;
			Screen.DrawHLine (_bbox.x + 2, _bbox.x + _bbox.xdim  - 2, y, WS_COLOR_BLACK);
		}
		y += boxHeight + 1;
	}
	int x = _bbox.x + 2 + boxWidth;
	for (int i = 1; i < 9; i++)
	{
		Screen.DrawVLine (x, _bbox.y + 2, _bbox.y + _bbox.ydim  - 2, WS_COLOR_BLACK);
		if (i % 3 == 0) {
			x++;
			Screen.DrawVLine (x, _bbox.y + 2, _bbox.y + _bbox.ydim  - 2, WS_COLOR_BLACK);
		}
		x += boxWidth + 1;
	}
}

void CtlSudoku::_DrawBorders()
{
}

void CtlSudoku::_DrawForeground()
{
	int boxHeight = (_bbox.ydim - 14) / 9;
	int boxWidth = (_bbox.xdim - 14) / 9;
	int yOffset = (boxHeight - SystemFont.GetHeight())/2;
	if (yOffset < 0) yOffset = 0;
	
	int y = _bbox.y + 2;
	for (int i = 0; i < 9; i++)
	{
		int x = _bbox.x + 2;
		for (int j = 0; j < 9; j++)
		{
			if (field[j][i].visibleValue > 0 && field[j][i].visibleValue < 10) //should NEVER be > 9
			{
				char b[2];
				sprintf (b, "%d", field[j][i].visibleValue);
				StringEx bla (b);
				u16 draw_Color;
				if (field[j][i].userSet && help && field[j][i].visibleValue == field[j][i].realValue) draw_Color = WS_COLOR_RED;
				else if (field[j][i].userSet) draw_Color = WS_COLOR_DARKGREY;
				else draw_Color = WS_COLOR_BLACK;
				
				if (this->done) draw_Color = WS_COLOR_RED;
				
				int xOffset = (boxWidth-bla.SizeEx())/2;
				if (xOffset < 0 || xOffset > 30) xOffset = 0;
				Screen.Printf (x + xOffset, y + yOffset, draw_Color, "%s", bla.c_str());
			}
			if (j % 3 == 0) x++;
			x += boxWidth + 1;
		}
		if (i % 3 == 0) y++;
		y += boxHeight + 1;
	}
	if (focus.x >= 0)
		Screen.DrawRectangle (fieldBbox(focus.x,focus.y).Inside(-1), WS_COLOR_RED);
}

Bbox CtlSudoku::fieldBbox (int x, int y)
{
	int xdim = (_bbox.xdim - 14) / 9;
	int ydim = 	(_bbox.ydim - 14) / 9;
	return Bbox (_bbox.x + x * xdim + ((x / 3 + 1) * 2) + x - (x / 3),
				_bbox.y + y * ydim + ((y / 3 + 1) * 2) + y - (y / 3),
				xdim, ydim);
/*	Bbox ret;
	ret.ydim = (_bbox.ydim - 14) / 9;
	ret.xdim = (_bbox.xdim - 14) / 9;
	ret.x = _bbox.x + x * ret.xdim + ((x / 3 + 1) * 2) + x - (x / 3);
	ret.y = _bbox.y + y * ret.ydim + ((y / 3 + 1) * 2) + y - (y / 3);
	return ret;*/
}
