#include <winsapi.h>
#include <vector>
#include "my_ctldroplist.h"
#include "cf_io/gba_nds_fat.h"

extern const unsigned short int cal_icon[];

typedef struct _DATE_
{
	int day_of_week;
	int day;
	int month;
	int year;
} DATE;

typedef struct _TIME_
{
	int hour;
	int minute;
} TIME;

typedef struct _GROUP_ENTRY_
{
	char name[32];
	u16 color;

} GROUP_ENTRY;

typedef struct _RECORD_DATA_
{
	DATE date;
	TIME start;
	TIME end;
	char task[256];
//	bool alarm;
	GROUP_ENTRY group;
	
} RECORD_DATA;

class ctl_limiter : public Control
{
	public:
		ctl_limiter(const Bbox &b, int c_start, int c_end, int c_increase_rate);
		int get_value() { return current_pos; }
		void set_value(int value) { current_pos = value; char temp[256]; sprintf(temp, "%d", current_pos); indicator->SetText(temp); }
		void set_end(int c_end) { end = c_end; indicator->UpdateAll(); }
		void set_start(int c_start) { start = c_start; indicator->UpdateAll(); }
		
	private:
		int start;
		int end;
		int increase_rate;
		int current_pos;
		CtlButtonEx* increase_button;
		CtlButtonEx* decrease_button;
		CtlLabel* indicator;
		
	protected: 
		void HandleMessages();
};

class back_colored_button : public CtlButton
{
	public:
		back_colored_button(const Bbox &box, char* label) : CtlButton(box, label) { }
		//helper functions
		void add_record(RECORD_DATA record) { data.push_back(record); }
		void clear_all() { data.clear(); UpdateAll(); }
		vector<RECORD_DATA > get_records() { return data; }
		int get_records_size() { return data.size(); }
		
	protected:
		void _DrawForeground();
		
	private:
		//globals
		vector<RECORD_DATA > data;
};

class add_edit_group;
class add_task;
class view_tasks;

class Organizer : public CtlWindow
{
	public:
		Organizer();
		void reset_groups(vector<GROUP_ENTRY > new_groups) { groups.clear(); groups = new_groups; fill_grid(current); }
		void add_record(RECORD_DATA record_data) { records.push_back(record_data); }
		void update_all() { fill_grid(current); UpdateAll(); }
		
	private:
		//globals
		int data_grid[7][6];
		DATE today;
		DATE current;
		vector<GROUP_ENTRY > groups;
		vector<RECORD_DATA > records;
		//helper functions
		void fill_grid(DATE time);
		DATE get_today_date();
		DATE add_days(DATE date, int days);
		DATE sub_days(DATE date, int days);
		DATE add_months(DATE date, int months);
		DATE sub_months(DATE date, int months);
		bool save_data();
		bool load_data();
		//controls
		back_colored_button* button_grid[7][6];
		back_colored_button* pressed_button;
		CtlButton *increase_button, *decrease_button;
		CtlTextBox *date_box;
		add_edit_group* group_edit_window;
		add_task* tasks_edit_window;
		view_tasks* view_tasks_window;
		CtlButton* tasks_button;
		CtlButton* groups_button;
		CtlButton* view_button;
		bool file_system;
		CtlButton* about_button;
	protected: 
		void HandleMessages();
};

class add_edit_group : public CtlWindow
{
	public:
		add_edit_group(Organizer* send_to);
		void set_working_groups(vector<GROUP_ENTRY > groups);
		
	private:
		//globals
		vector<GROUP_ENTRY > work_groups;
		//controls
		CtlButton* accept_button;
		CtlButton* cancel_button;
		CtlButton* delete_button;
		CtlEditBox* edit_name;
		my_CtlDropList* group_list;
		Control* selected_color;
		CtlSlider *red;
		CtlSlider *green;
		CtlSlider *blue;
		Organizer* my_ancestor;
		
	protected:
		void HandleMessages();
};


class add_task : public CtlWindow
{
	public: 
		add_task(vector<GROUP_ENTRY > current_groups, Organizer* send_to);
		void set_work_groups(vector<GROUP_ENTRY > c_groups);
		void set_date(DATE date);
		void reset_all();
		
	private:
		//globals
		vector<GROUP_ENTRY > work_groups;
		DATE work_date;
		Organizer* my_ancestor;
		//controls
		ctl_limiter *day, *month, *year;
		ctl_limiter *hour_start, *minute_start;
		ctl_limiter *hour_end, *minute_end;
		my_CtlDropList *groups;
		CtlEditBox* comments;
		CtlButton* add_button;
		CtlButton* cancel_button;
	protected:
		void HandleMessages();
};


class OrganizerApp : public WinSApp
    {
    public:
        OrganizerApp():WinSApp("Organizer", cal_icon){}
		
        void Launch() const;
};

class view_tasks : public CtlWindow
{
	public: 
		view_tasks(Organizer* send_to, back_colored_button* data_button);
		void set_button(back_colored_button* data_button) { data = data_button->get_records(); update_all(); }
		void update_all();
		
	private:
		Organizer* my_ancestor;
		my_CtlDropList* items;
		CtlLabel* group_label;
		CtlTextBox* date_box;
		CtlTextBox* comment;
		CtlButton* done_button;
		vector<RECORD_DATA > data;
		
	protected:
		void HandleMessages();
};
