/*! \class CtlDropList
    \brief A Drop down list
    \ingroup WinS

A CtlDropList is an object sending a WS_MSG_DROPDOWNLISTCHOICE message to its ancestor whenever
it's changes it's value. A drop down list can contain as meny items as you want
and it can be used for a menu with multiple choices. Example : 

\code
FORALLMESSAGES(Msg)
{
	switch (Msg.id)
	{
		case WS_MSG_DROPDOWNLISTCHOICE:
			if( Msg.data == -1 ) // None of the tabs are selected
			{ 
			}
			if( Msg.data == 0 ) // First item has been selected
			{ 
			}
			else if( Msg.data == 1 ) // Second item has been selected
			{ 
			}
			else if( Msg.data == 2 ) // Third item has been selected
			{ 
			}
				// and so on
			break;
	}
}
\endcode
*/

#ifndef _MY_CTL_DDLIST_INCLUDED_
#define _MY_CTL_DDLIST_INCLUDED_

#include <WinSAPI.h>
#include <wgl.h>

class my_CtlDropList : public Control
{
public:
	//! The constructor
	/*! \param b The Control's Bbox*/
	my_CtlDropList(const Bbox &b);
	
	/*! The Constructor usign a user's choice font.
		Please note that the height of the drop down list will
		be changed using the font's height
		\param b The control's Bbox
		\param f The Dropdownlist's Bbox */
	my_CtlDropList(const Bbox &b, Font& f);
		
	//! Adds an item to the drop down list
	/*! \param s the Text of the tobe added item*/
	void AddItem(const char *s);
	
	//! Returns the currect selected item
	/*! \return the currect selected item, or -1 otherwise */
	int GetSelectedItem() { return _currentitem; }

	//! Cahnge the currect selected item
	/*! \param item the item's number*/
	void SetItem(int item) { _currentitem = item; UpdateAll(); }

	//! Cahnge the currect font
	/*! \param f The font to change to*/
	void SetFont(Font& f);
	
	//! Retreive the number of labels in the drop list
	/*! \return The number of items in the drop list*/
	int GetItemCount() { return _labels.size(); }
	
	//! Removes an item from the drop down list.
	/*! \param n The number of the item to be removed */
	void RemoveItem(int n);
	
	//! Sets the maximum number of items for use with a scroller. default is 4
	/*! \param n the limit of items */
	void SetScrollerLimit(int n) { limitScroller=n; UpdateAll(); }
	
	//! Gets the maximum number of items for use with a scroller.
	/*! \return the number of items to limit */
	int GetScrollerLimit() { return limitScroller; }
	
	//! Clears all of the data from the DropList
	void ClearAll();
	
protected:
    void _DrawForeground();
	void _DrawBackground();
    void HandleMessages();
	
private:
	void _Render(Bbox vSize);
	void drawButton(bool isPressed, Bbox butBox, char label);
	int GetItemAt(int x, int y);
	vector<string> _labels;
	int _currentitem;
	bool reverse;
	Font _font;
	CtlButtonEx *_OpenButton;
	int curItemPrint;
	bool isGreather;
	int limitScroller;
};

#endif

