#include <stdlib.h>
#include <string>

#include <winsapi.h>

#include "meds.h"

/* Entry codes */
#define MODE        1
#define CASE        2
#define SPACE       3

/* Button display */
const char cButtonTexts[][12][10] =
{
    { "    A   V",
      "    N  L ",
      "    I X  ",
      "    HK # ",
      "QUPCOBGDJ",
      " ^ MR    ",
      "  Y T    ",
      " W  EZ!?;",
      "F   S    ",
      "   DEL   ",
      "     _   ",
      "   _     " },
    { "    a   v",
      "    n  l ",
      "    i x  ",
      "    hk # ",
      "qupcobgdj",
      " ^ mr    ",
      "  y t    ",
      " w  ez,.:",
      "f   s    ",
      "   DEL   ",
      "     _   ",
      "   _     " },
    { "    1    ",
      "    2    ",
      "    3    ",
      "    4  # ",
      "    5    ",
      "    6    ",
      "    7    ",
      "    8    ",
      "    9    ",
      "   DEL   ",
      "    0    ",
      "    _    " },
    { "    1    ",
      "    2    ",
      "    3    ",
      "    4  # ",
      "    5    ",
      "    6    ",
      "    7    ",
      "    8    ",
      "    9    ",
      "   DEL   ",
      "    0    ",
      "    _    " }
};

/* Button codes */
const char cButtonCodes[][12][10] =
{
    { "    A   V",
      "    N  L ",
      "    I X  ",
      "    HK \1 ",     // \1 == MODE switch
      "QUPCOBGDJ",
      " \2 MR    ",     // \2 == CASE switch
      "  Y T    ",
      " W  EZ!?;",      // \n == New line
      "F   S    ",
      "    \b    ",     // \b == Backspace
      " \n\n\3\3    ",  // \3 == Space
      "\n\n \3\3    " },
    { "    a   v",
      "    n  l ",
      "    i x  ",
      "    hk \1 ",     // \1 == MODE switch
      "qupcobgdj",
      " \2 mr    ",     // \2 == CASE switch
      "  y t    ",
      " w  ez,.:",      // \n == New line
      "f   s    ",
      "    \b    ",     // \b == Backspace
      " \n\n\3\3    ",  // \3 == Space
      "\n\n \3\3    " },
    { "    1    ",
      "    2    ",
      "    3    ",
      "    4  \1 ",     // \1 == MODE switch
      "    5    ",
      "    6    ",
      "    7    ",
      "    8    ",
      "    9    ",
      "    \b    ",     // \b == Backspace
      "    0    ",
      "\n\n  \3    " }, // \3 == Space
    { "    1    ",
      "    2    ",
      "    3    ",
      "    4  \1 ",     // \1 == MODE switch
      "    5    ",
      "    6    ",
      "    7    ",
      "    8    ",
      "    9    ",
      "    \b    ",     // \b == Backspace
      "    0    ",
      "\n\n  \3    " }  // \3 == Space
};

wMedsWindow::~wMedsWindow()
{
    /* Get rid of our output window */
    if(messageWindow)
    {
        messageWindow->Destroy();
    };
};

wMedsWindow::wMedsWindow(Bbox b):
    CtlWindow(b, "MEDS", "")
{
    int i = 0;
    
    currentMode = MODE_CHARS;
    currentCase = CASE_LOWER;
    
    /* Buttons */
    for(int y = 0; y < 4; y++)
    {
        for(int x = 0; x < 3; x++)
        {
            gsButtons[i] = new CtlButtonCn(Bbox(x * MEDS_BUTTON_X, y * MEDS_BUTTON_Y,
                                           MEDS_BUTTON_X, MEDS_BUTTON_Y));
            
            if(!gsButtons[i])
            {
                delete this;
                return;
            };
            
            gsButtons[i]->setFonts((Font &)SmallFont);
            gsButtons[i]->SetStyle(FLAT);
            
            gsButtons[i]->SetBackColor(i & 1 ? RGB15(0, 0, 26) : RGB15(0, 0, 17));
            gsButtons[i]->SetForeColor(RGB15(28, 28, 23));
            
            gsButtons[i]->xborder = 1;
            gsButtons[i]->yborder = 1;
            gsButtons[i]->AutoResizeX(true);
            gsButtons[i]->AutoResizeY(true);
            
            AddSubControl(gsButtons[i]);
            
            i++;
        };
    };
    
    doMode();
    
    pressbutton = 0;
    releasebutton = 0;
    
    /* Message window */
    messageWindow = new CtlConsoleWindow();
    
    if(!messageWindow)
    {
        delete this;
        return;
    };
    
    messageWindow->getTextBox()->SetForeColor(0x8000);
    messageWindow->getTextBox()->SetBackColor(0xFFFF);
    messageWindow->getTextBox()->SetStyle(LIGHT_INNER);
    
    gWinS.AddSubControl(messageWindow);
};

int wMedsWindow::handleInput(int press, int release)
{
    int ret;
    
    if(press > 11
    || press < 0
    || release > 11
    || release < 0)
    {
        return(-1);
    };
    
    ret = cButtonCodes[getButtonSetNum()][press][release - press + 4];
    
    if(ret == ' ')
        ret = '\0';
    else if(ret == SPACE)
        ret = ' ';
    
    return(ret);
};

void wMedsWindow::doMode()
{
    for(int i = 0; i < 12; i++)
    {
        gsButtons[i]->setTexts(cButtonTexts[getButtonSetNum()][i]);
    };
};

int wMedsWindow::getButtonSetNum()
{
    return(currentCase + (currentMode << 1));
};

void wMedsWindow::HandleMessages()
{
    WS_MSG msg;
    
    FORALLMESSAGES(msg)
    {
        switch(msg.id)
        {
            case WS_MSG_POINTERINPUT:
                int x, y;
                
                gWinS.GetTouch(x, y);
                
                pressbutton = -1;
                releasebutton = -1;
                
                for(int i = 0; i < 12; i++)
                {
                    if(gsButtons[i]->GetClientArea().IsContained(x, y))
                    {
                        pressbutton = i;
                        releasebutton = i;
                        
                        break;
                    };
                };
                
                while(gWinS.InputReceived())
                {
                    int nx, ny;
                    
                    gWinS.GetTouch(nx, ny);
                    
                    if(GetClientArea().IsContained(nx, ny))
                    {
                        x = nx;
                        y = ny;
                    };
                };
                
                for(int i = 0; i < 12; i++)
                {
                    if(gsButtons[i]->GetClientArea().IsContained(x, y))
                    {
                        releasebutton = i;
                        
                        break;
                    };
                };
                
                int result = handleInput(pressbutton, releasebutton);
                
                switch(result)
                {
                    case '\b':      /* Backspace */
                        messageWindow->del(1);
                        
                        break;
                    case CASE:      /* Case switch */
                        if(currentCase == CASE_UPPER)
                            currentCase = CASE_LOWER;
                        else
                            currentCase = CASE_UPPER;
                        
                        doMode();
                        
                        break;
                    case MODE:      /* Mode switch */
                        if(currentMode == MODE_CHARS)
                            currentMode = MODE_NUMER;
                        else
                            currentMode = MODE_CHARS;
                        
                        doMode();
                        
                        break;
                    case '\0':      /* Nothing */
                        break;
                    case -1:        /* Error */
                        break;
                    default:
                        char write[2] = {0, 0};
                        
                        write[0] = result;
                        messageWindow->write(string(write));
                        
                        break;
                };
                
                break;
        };
    };
    
    CtlWindow::HandleMessages();
};

/* EOF */
